/***************************************************************************\
 *  Name    : dds_7032.ino                                                *
 *  Author  : Beric Dunn (K6BEZ)  / Dk2jk Heribert schulte                   *                                                         *
 *  Notes   : Written using for the Arduino Nano                             *
 *          :   Pins:                                                        *
 *          :    A0 - Reverse Detector Analog in                             *
 *          :    A1 - Forward Detector Analog in
 \***************************************************************************/

// Define Pins used to control AD9850 DDS

// board von hermann 13.dez.2014
const int SCLK  = 12;
const int FQ_UD = 11;
const int SDAT  = 10;
const int RESET = 9;

double Fstart_MHz = 7.032;  // Start Frequency for sweep
//double Fstart_MHz = 0.0;  // Start Frequency for sweep
double current_freq_MHz ; // Temp variable used during sweep
const double referenz_voltage = 1100.0; // mV
double offset_revers = 0;
double offset_forward = 0;

// the setup routine runs once when you press reset:
void setup() {
  // Configiure DDS control pins for digital output
  pinMode(FQ_UD, OUTPUT);
  pinMode(SCLK, OUTPUT);
  pinMode(SDAT, OUTPUT);
  pinMode(RESET, OUTPUT);

  // Configure LED pin for digital output
  pinMode(13, OUTPUT);


  // Set up analog inputs on A0 and A1, internal reference voltage
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  //analogReference(DEFAULT);
  analogReference(INTERNAL);

  // initialize serial communication at 57600 baud
  Serial.begin(57600);


  // Reset the DDS
  digitalWrite(RESET, HIGH);
  digitalWrite(RESET, LOW);

}

// the loop routine runs over and over again forever:
void loop() {
  double FWD = 0;
  double REV = 0;
  double VSWR;

  //Turn frequency into FStart and set DDS output to single frequency
  //Fstart_MHz = ((double)serial_input_number)/1000000;
  //SetDDSFreq(Fstart_MHz);
  SetDDSFreq(Fstart_MHz * 1000000);
  // Wait a little for settling
  delay(10);
  // Read the forawrd and reverse voltages
  REV = (double)analogRead(A0) - offset_revers; //* referenz_voltage /1024.0;
  FWD = (double)analogRead(A1) - offset_forward; // * referenz_voltage /1024.0;
// REV = REV - 13; //korrektur?
//  if (REV >= FWD) {
//    REV = FWD - 1;
//  }
//  if (REV < 1) {
//    REV = 1;
//  }


  VSWR = (FWD + REV) / (FWD - REV);
  Serial.print  ( "   VSWR = ");
  Serial.println  ( VSWR , 3);
  Serial.print  ( "forward = ");
  Serial.print  ( FWD, 0);
  Serial.println  (" digits");
  Serial.print  ( "reverse = ");
  Serial.print  ( REV, 0);
  Serial.println  (" digits");
  delay(1000);

}

void SetDDSFreq(double Freq_Hz) {
  // Calculate the DDS word - from AD9850 Datasheet
  int32_t f = Freq_Hz * 4294967295 / 125000000;
  // Send one byte at a time
  for (int b = 0; b < 4; b++, f >>= 8) {
    send_byte(f & 0xFF);
  }
  // 5th byte needs to be zeros
  send_byte(0);
  // Strobe the Update pin to tell DDS to use values
  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);
}

void send_byte(byte data_to_send) {
  // Bit bang the byte over the SPI bus
  for (int i = 0; i < 8; i++, data_to_send >>= 1) {
    // Set Data bit on output pin
    digitalWrite(SDAT, data_to_send & 0x01);
    // Strobe the clock pin
    digitalWrite(SCLK, HIGH);
    digitalWrite(SCLK, LOW);
  }
}

